Package: Uniquorn
Title: Identification of cancer cell lines based on their weighted
        mutational/ variational fingerprint
Version: 2.33.0
Authors@R: person("Raik", "Otto", email = "raik.otto@hu-berlin.de", role = c("aut", "cre"))
Description: 'Uniquorn' enables users to identify cancer cell lines.
 Cancer cell line misidentification and cross-contamination reprents a significant challenge for cancer researchers. 
 The identification is vital and in the frame of this package based on the locations/ loci of somatic and germline mutations/ variations.
 The input format is vcf/ vcf.gz and the files have to contain a single cancer cell line sample 
 (i.e. a single member/genotype/gt column in the vcf file).
Imports: stringr, R.utils, WriteXLS, stats, doParallel, foreach,
        GenomicRanges, IRanges, VariantAnnotation, data.table
Depends: R (>= 3.5)
License: Artistic-2.0
Type: Package
Date: 2025-10-22
Author: Raik Otto
RoxygenNote: 7.2.3
NeedsCompilation: no
Suggests: testthat, knitr, rmarkdown, BiocGenerics
biocViews: ImmunoOncology, StatisticalMethod, WholeGenome, ExomeSeq
VignetteBuilder: knitr
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/Uniquorn
git_branch: devel
git_last_commit: f9ade8c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
Packaged: 2026-05-18 23:45:39 UTC; biocbuild
Maintainer: Raik Otto <raik.otto@hu-berlin.de>
Built: R 4.6.0; ; 2026-05-19 05:13:14 UTC; unix

# Generated by roxygen2: do not edit by hand

export(add_custom_vcf_to_database)
export(identify_vcf_file)
export(initiate_canonical_databases)
export(read_library_names)
export(remove_ccls_from_database)
export(remove_library_from_database)
export(show_contained_ccls)
export(show_contained_variants_for_ccl)
export(show_contained_variants_in_library)
export(show_which_ccls_contain_variant)
import(GenomicRanges)
import(IRanges)
import(R.utils)
import(WriteXLS)
import(foreach)
import(stringr)
importFrom(IRanges,IRanges)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,subsetByOverlaps)
importFrom(VariantAnnotation,readVcf)
importFrom(data.table,fread)
importFrom(stats,aggregate)
importFrom(stats,integrate)
importFrom(stats,pbeta)
importFrom(stats,pbinom)

#' init_and_load_identification
#' 
#' Initiate the analysis
#' Output basic information
#' 
#' \code{init_and_load_identification} parses vcf file and output 
#' basic information
#' @param verbose Print additional information
#' @param ref_gen Reference genome version. All training sets are 
#' associated with a reference genome version. Default: GRCH37
#' @param vcf_file Path to vcf_file
#' @param output_dir Output directory for identification results
#' @usage 
#' init_and_load_identification( 
#'     verbose,
#'     ref_gen,
#'     vcf_file,
#'     output_dir
#' )
#' @return Three file path instances and the fingerprint
init_and_load_identification = function(
    verbose,
    ref_gen,
    vcf_file,
    output_dir
){
    
    vcf_fingerprint = parse_vcf_file(vcf_file)
    vcf_file_name = tail(as.character(unlist(strsplit(vcf_file, "/"))), 1)
    
    if (output_dir == ""){
        output_file = base::paste(vcf_file, "uniquorn_ident.tab", sep = "_")
        
    } else if (base::dir.exists(output_dir)){
        output_file = base::paste(
            output_dir,
            base::paste( 
                vcf_file_name,
                "uniquorn_ident.tab",
                sep = "_"
            ),
            sep = "/"
        ) 
    }
    output_file_xls = base::gsub(".tab$", ".xls", output_file)
    
    res_list = list( 
        "output_file"     = output_file,
        "output_file_xls" = output_file_xls,
        "vcf_fingerprint" = vcf_fingerprint,
        "vcf_file_name"   = vcf_file_name
    )
    
    return(res_list)
}

#' add_missing_cls
#' 
#' @param res_table Table that contains the identification results
#' @param dif_cls Missing CLs
#' @return Results table with added missing cls
add_missing_cls = function(
    res_table,
    dif_cls
){

    for (cl in dif_cls){

        res_table = data.frame(
            "CL" = c(as.character(res_table$CL), 
                as.character(cl)),
        "CL_source" = c( as.character( res_table$CL_source),
            utils::tail(unlist(str_split(cl,"_")), 1
        )),
        "Found_muts_abs" = c(as.character( res_table$Found_muts_abs ),
            as.character( "0" )),
        "Count_mutations_abs" = c(as.character(res_table$Count_mutations_abs),
            as.character( "0" )),
        "P_values"    = c( as.character( res_table$P_values),
                            as.character( "1" )),
        "Q_values"    = c( as.character( res_table$Q_values),
                            as.character( "1" )),
        "P_value_sig" = c( as.character( res_table$P_value_sig),
                            as.character( "FALSE" )),
        "Q_value_sig" = c( as.character( res_table$Q_value_sig),
                            as.character( "FALSE" ))
        )
    }
    return(res_table)
}

#' Filter Parsed VCF Files
#' 
#' Intern utility function. Filters the parsed VCF file for all informations
#' except for the start and length of variations/mutations.
#' 
#' @param vcf_file character string giving the path to the vcf file
#'  on the operating system.
#' @param ref_gen Reference genome version
#' @param library_name Name of the reference library
#' @usage 
#' parse_vcf_file(
#'     vcf_file,
#'     ref_gen,
#'     library_name
#' )
#' @import GenomicRanges stringr
#' @importFrom IRanges IRanges
#' @importFrom VariantAnnotation readVcf
#' @return Loci-based DNA-mutational fingerprint of the cancer cell line
#'  as found in the input VCF file.
parse_vcf_file = function(
    vcf_file,
    ref_gen,
    library_name
){
    switch(ref_gen,
        "GRCH37" = {ref_gen = "hg19"}
    )
    
    g_query =  VariantAnnotation::readVcf(
      file = vcf_file
    )
    
    # process variants
    chroms = stringr::str_replace(as.character(seqnames(g_query)),
                         pattern = "chr|CHR", "")
    start_var = start(g_query)
    end_var = start_var + width(g_query) - 1
    
    chroms_pure = grep(chroms, pattern = "_", invert = TRUE)
    chroms      = chroms[chroms_pure]
    chroms      = str_replace(chroms, pattern = "^chr", "")
    start_var   = start_var[chroms_pure]
    end_var     = end_var[chroms_pure]
    
    cl_id = gsub("^.*/", "", vcf_file)
    cl_id = gsub(".vcf", "", cl_id, fixed = TRUE)
    cl_id = gsub(".VCF", "", cl_id, fixed = TRUE)
    cl_id = gsub(".hg19", "", cl_id, fixed = TRUE)
    cl_id = toupper(cl_id)
    cl_id = str_replace_all(cl_id, pattern = "\\.", "_")
    cl_id = paste0(cl_id, "_", library_name)
    
    # build fingerprint and return
    g_query = GenomicRanges::GRanges(
        seqnames = chroms,
        ranges = IRanges(start = start_var, end = end_var),
        Member_CCLs = rep(cl_id, length(g_query))
    )

    return(g_query)
}

write_w0_and_split_w0_into_lower_weights = function(
    g_mat,
    library_name,
    ref_gen
){
    
    ccl_stats <<- data.frame(
        "CCL" = names(table(as.character(
            unlist(str_split(g_mat$Member_CCLs, pattern = ","))))),
        "Mut_count_0" = as.character(table(as.character(unlist(
            str_split(g_mat$Member_CCLs, pattern = ",")))))
    )
    
    if ( "Member_CCLs" %in% names(GenomicRanges::mcols(g_mat)) ){
        
        member_count = as.integer( str_count(g_mat$Member_CCLs, pattern = ","))
        member_keep_index    = which( member_count < 10 )
        member_exclude_index = which( member_count >= 10 )
        
        g_mat_exclude <<- g_mat[member_exclude_index]
        g_mat         <<- g_mat[member_keep_index]
        
        write_mutation_grange_objects(
            mutational_weight_inclusion_threshold = 0,
            g_mat = g_mat_exclude,
            library_name = library_name,
            ref_gen = ref_gen,
            type = "excluded_variants"
        )
    }
    
    write_mutation_grange_objects(
        mutational_weight_inclusion_threshold = 0,
        g_mat = g_mat,
        library_name = library_name,
        ref_gen = ref_gen
    )
    
    for(mwit in c(.25,.5,1.0)){
        
        if ( !(  "Member_CCLs" %in% names(GenomicRanges::mcols(g_mat)) )){
            
            hit_index = 0
            
        } else {
            
            hit_index = which( str_count(
                GenomicRanges::mcols(g_mat)$Member_CCLs, pattern = ","
                ) <= as.integer( round(1/mwit) ) )
        }
        
        mwit_g_mat = g_mat[hit_index]
        
        write_mutation_grange_objects(
            mutational_weight_inclusion_threshold = mwit,
            g_mat = mwit_g_mat,
            library_name = library_name,
            ref_gen = ref_gen
        )
        
        mut_t = table(as.character(unlist(str_split( 
            GenomicRanges::mcols(mwit_g_mat)$Member_CCLs,pattern = "," ))))
        
        ccl_count = rep("0", nrow(ccl_stats))
        ccl_match = match( ccl_stats$CCL, names(mut_t), nomatch = 0 )
        ccl_count[ccl_match] = mut_t[ccl_match != 0]
        ccl_stats = cbind(ccl_stats, ccl_count)
    }
    colnames(ccl_stats) = c("CCL","W0","W25","W05","W1")
    
    package_path = system.file("", package = "Uniquorn")
    
    input_ccls = unique(as.character(unlist(
        str_split( g_mat$Member_CCLs, pattern = "," ) ) ))
    output_ccls = unique(as.character(unlist(
        str_split( ccl_stats$CCL, pattern = "," ) ) ))
    missing_ccls = sum((input_ccls %in%  output_ccls) == FALSE)
    
    if( missing_ccls != 0) stop("Loading of CCLs has not worked!")
    
    CCL_stats_data_path =  paste( c(
        package_path,"/Libraries/",
        ref_gen,"/",library_name,
        "/CCL_List_Uniquorn_DB.rds"),
        sep ="", collapse= ""
    )
    saveRDS(ccl_stats,CCL_stats_data_path)
}

#' add_p_q_values_statistics
#' 
#' A hypergeometric distribution-assumption allows to calculate the 
#' p-values for a significant or non-significant overlap in this function
#' 
#' \code{add_p_q_values_statistics} Calculates the p-values
#' 
#' @param g_query IRanges object that contains the query variants
#' @param match_t A table that contains the nubmber of matching variants
#' @param p_value Threshold for the significance p-value
#' @param ref_gen Reference genome version
#' @param minimum_matching_mutations Manual lower amount of matching
#' mutations require for a significant match between a query and a 
#' reference
#' @param top_hits_per_library limits significant similarities to the first 
#' n hits
#' @import stringr
#' @importFrom stats pbinom
#' @return R table with a statistic 
add_p_q_values_statistics = function(
    g_query,
    match_t,
    p_value,
    ref_gen,
    minimum_matching_mutations,
    top_hits_per_library
){
    
    library_names = read_library_names(ref_gen = ref_gen)
    p_values = rep(1.0, nrow(match_t))
    balls_in_query = length(g_query$Member_CCLs)
    sig_vec <<- c()
    
    for (library_name in library_names){
        
        index_library = which( match_t$Library == library_name  )
        white_balls_possible = as.integer(as.character(
            match_t$All_variants))[index_library]
        
        if( length(white_balls_possible) == 0){
            stop("There is only one CCL in the custom set.",  
                "A confidence score calculation is only ", 
                "possible with more than one sample!")
        }
        
        # matching features
        white_balls_found = as.integer(
            as.character(match_t$Matches))[index_library]
        white_balls_found_least_one = sum(white_balls_found > 0)
        
        # not matching features
        black_balls = sum(white_balls_possible) - white_balls_possible
        
        # estimation of 'noise'
        background_cls_traces = sum(
            white_balls_found >= mean(white_balls_found)
        )
        
        # here, we apply a heuristic on the likelihood to match
        likelihood_found = white_balls_possible / sum(white_balls_possible)

        q = white_balls_found - 1
        q[q < 0]   = 0
        
        # here we calculate the p-value for the amount of matches
        p_values_panel = as.double(stats::pbinom(
            q = q,
            size = sum(white_balls_found),
            p = likelihood_found,
            lower.tail = FALSE 
        ))
        
        p_values_panel[white_balls_found == white_balls_possible] = 0.0
        p_values_panel[p_values_panel >= 1.0] = 1.0
        p_values_panel[white_balls_found == 0] = 1.0
        p_values_panel[white_balls_found < minimum_matching_mutations] = 1.0
        
        p_values[index_library] = p_values_panel
        n_hits_vec = order( as.double( p_values[index_library] ) )
        n_hits_vec[as.integer(n_hits_vec) >
            as.integer(top_hits_per_library)] = FALSE
        n_hits_vec[as.integer(n_hits_vec) > 0 ] = TRUE
        n_hits_vec = as.logical(n_hits_vec)
        
        sig_vec <<- c(sig_vec, n_hits_vec)
    }
    
    match_t$P_values = p_values
    match_t$P_value_sig = match_t$P_values <= p_value
    match_t$P_value_sig = match_t$P_value_sig & sig_vec

    return(match_t)
}

#' add_penalty_statistics
#' 
#' Add penalty statistics to results
#' 
#' @param match_t object that contains the matching variants
#' @param minimum_matching_mutations a numerical giving the minimum amount of
#'  mutations that has to match between query and training sample for a 
#'  positive prediction
#' @importFrom stats integrate
#' @importFrom stats pbeta
#' @return The updated statistics
add_penalty_statistics = function(match_t, minimum_matching_mutations){

    matching_variants = as.integer(match_t$Matches)
    minimum_matching_mutations = min(matching_variants)
    matching_variants = matching_variants[ matching_variants > 0]
    
    if ( length(matching_variants) == 0 ){
        
        # initalize penality values -> defensive programming
        penalty = 0
        penalty_mutations = 0
        
    } else{ 
        
        # these are the two parameter relevant for generalization
        mean_match = mean( matching_variants )
        max_match  = max( matching_variants )
        
        # here, we utilize the conjugate prior beta distribution
        penalty = integrate(
            f = pbeta,
            0,
            1,
            max_match,
            max_match/ mean_match,
            stop.on.error = FALSE
            )$value
        
        # this is the precise amount of penality applied    
        penalty_mutations = ceiling(
            mean_match + (max_match * penalty) / (1 - penalty)
        )
    }
    
    if ( ( minimum_matching_mutations == 0 ) & ( penalty != 0.0) ){
        
        message(
            "Correcting the background due to traces of random, ", 
            "scale-freeness amounts of matches, requiring at least ", 
            as.character(penalty_mutations), " variants to match."
        )
    }
    match_t$Above_Penalty = 
        as.integer(as.character(match_t$Matches)) > penalty_mutations
    
    return(match_t)
}

#' create_bed_file
#' 
#' Creates BED files from the found and not found annotated mutations
#' 
#' @param match_t R table which contains the mutations from the 
#' training database for the cancer cell lines
#' @param vcf_fingerprint contains the mutations that are present 
#' in the query cancer cell line's vcf file
#' @param output_file Path to output file
#' @param ref_gen Reference genome version
#' @param manual_identifier Manually enter a vector of CL name(s) 
#' whose bed files should be created, independently from them 
#' passing the detection threshold
#' @return Returns a message which indicates if the BED file 
#' creation has succeeded
#' @usage 
#' create_bed_file(
#' match_t,
#' vcf_fingerprint,
#' output_file,
#' ref_gen,
#' manual_identifier
#' 
#' )
create_bed_file = function( 
    match_t,
    vcf_fingerprint,
    output_file,
    ref_gen,
    manual_identifier
){
    
    # prep
    
    # start
    
    print("Creating bed files")
    
    found_res_tab = res_table[ as.logical( res_table$Conf_score_sig ), ]
    found_identifier = paste(
        found_res_tab$CL, found_res_tab$CL_source, sep = "_"
    )
    
    found_identifier = unique( c( manual_identifier, found_identifier ) )
    
    for ( identifier in found_identifier ){
        
        if ( identifier != "" ){
            
            print(identifier)
            
            name_training_bed_file = stringr::str_replace(
                output_file,
                "_uniquorn_ident.tab",
                paste0( 
                    c(
                        "_uniquorn_ident_mutations_reference_variants_",
                        identifier,
                        ".bed"
                    ),
                    collapse = ""
                )
            )
            name_query_bed_file    = stringr::str_replace(
                output_file, "_uniquorn_ident.tab", 
                    paste0(
                        c(
                            "_uniquorn_ident_mutations_query_variants_",
                            identifier,
                            ".bed"
                        ),
                        collapse = ""
                    )
            )
            name_missed_bed_file = stringr::str_replace(
                output_file,
                "_uniquorn_ident.tab",
                paste0(
                    c(
                        "_uniquorn_ident_mutations_non_matching_variants_",
                        identifier,
                        ".bed"
                    ),
                    collapse = ""
                )
            )
            
            # training
            training_bed_file = paste0( 
                c( 
                    'track name=',
                    'Training_',
                    identifier,
                    ' description=Variants_in_reference type=bedDetail db=',
                    ref_gen,
                    ' color=0,0,255 priority=3'
                ),
                collapse = ""
            )
            
            training_coords = sim_list$Fingerprint[ sim_list$CL == identifier ]
            training_coords = stringr::str_trim( training_coords )
            training_coords = training_coords[
                !grepl("^[c(']", training_coords)]
            training_coords = stringr::str_split( training_coords, "_" )
            
            training_coords_res = sapply(training_coords, 
                FUN = function( vec ){ 
                    chrom = paste( "chr", stringr::str_trim(vec[1]), sep = "" )
                    return( 
                        paste0( 
                            c(
                                chrom,
                                as.character(
                                    as.integer(
                                        vec[2]
                                    ) - 1
                                ),
                                as.integer(
                                    as.integer(
                                        vec[3]
                                    )
                                )
                            ),
                            collapse = "\t"
                        )
                    )
                }
            )
            training_coords_res = c(
                training_bed_file,
                training_coords_res
            )
            
            utils::write.table( 
                x = training_coords_res,
                file =  name_training_bed_file,
                sep ="",
                row.names = FALSE,
                col.names = FALSE,
                quote = FALSE
            )
    
            # query
            
            query_bed_file = paste0( 
                c( 
                    'track name=',
                    'Query_',
                    identifier,
                    ' description=Variants_in_query type=bedDetail db=',
                    ref_gen,
                    ' color=0,255,0 priority=3'
                ),
                collapse = ""
            )
            
            query_coords = vcf_fingerprint
            query_coords = stringr::str_trim( query_coords )
            query_coords = query_coords[ ! grepl("^[c(']", query_coords ) ]
            query_coords = stringr::str_split( query_coords, "_" )
            
            query_coords_res = sapply( query_coords, FUN = function( vec ){ 
                chrom = paste( "chr", stringr::str_trim( vec[1] ), sep = "" )
                return( 
                    paste0(
                        c(
                            chrom,
                                as.character(
                                    as.integer(
                                        vec[2]
                                    ) - 1 
                                ),
                                as.integer(
                                    as.integer(
                                        vec[3]
                                    )
                                )
                        ),
                        collapse = "\t"
                    )
                )
            } )
            query_res = c( query_bed_file, query_coords_res )
            utils::write.table( 
                x = query_res,
                file =  name_query_bed_file,
                sep = "",
                row.names = FALSE,
                col.names = FALSE,
                quote     = FALSE 
            )        
                    
            # missed
            missed_bed_file = paste0( 
                c( 
                    'track name=',
                    'Missed_',
                    identifier,
                    ' description=Non_matching_variants type=bedDetail db=',
                    ref_gen,
                    ' color=255,0,0 priority=3'
                ),
                collapse = ""
            )
            
            missed_coords = training_coords[
                which( ! ( training_coords %in% query_coords ) ) ]
            
            missed_coords = sapply( missed_coords, FUN = function( vec ){ 
                chrom = paste( "chr", stringr::str_trim( vec[1] ), sep = "" )
                return(
                    paste0(
                        c(
                            chrom,
                                as.character(
                                    as.integer(
                                        vec[2]
                                    ) -1
                                ),
                                as.integer(
                                    as.integer(
                                        vec[3]
                                    )
                                )
                        ),
                        collapse = "\t"
                    )
                )
            })
            missed_coords_res = c( missed_bed_file, missed_coords )
            
            utils::write.table(
                x = missed_coords_res,
                file =  name_missed_bed_file,
                sep = "",
                row.names = FALSE,
                col.names = FALSE,
                quote = FALSE
            )
        }
    }
}

#' show_contained_ccls
#' 
#' This function displays the names, amount of mutations and the overall 
#' weight of the mutations of all contained cancer cell line fingerprints 
#' for a chosen reference genome and optional library.
#' 
#' @param ref_gen a character vector specifying the reference genome version.
#'  All training sets are associated with a reference genome version.
#'  Default is \code{"GRCH37"}.
#' @param verbose Should DB informations be printed?
#' @return R table which contains identifiers of all cancer cell line samples
#'  which match the specified parameters (reference genome and library).
#' @usage 
#' show_contained_ccls(ref_gen, verbose)
#' @examples
#' ## Show all contained cancer cell lines for reference GRCH37:
#' show_contained_ccls(ref_gen = "GRCH37", verbose = TRUE)
#' @import GenomicRanges stringr
#' @export
show_contained_ccls = function(
    ref_gen = "GRCH37",
    verbose = TRUE
){
    
    package_path = system.file("", package = "Uniquorn")
    library_path =  paste( c( package_path,"/Libraries/",ref_gen),
        sep ="", collapse= "")
    
    if ( ! dir.exists( library_path ))
        stop("No libraries found!")
    
    libraries = list.dirs(library_path, full.names = FALSE)
    libraries = libraries[ libraries != ""]
    
    ccls_all <<- data.frame(
        "CCL" = as.character(),
        "Library" = as.character(),
        "W0" = as.character(),
        "W25" = as.character(),
        "W05" = as.character(),
        "W1" = as.character()
    )
    
    for (library_name in libraries){
        
        stats_path = paste( c( library_path,"/",library_name,
            "/CCL_List_Uniquorn_DB.rds"), sep ="", collapse= "")
        g_library = readRDS(stats_path)
        cl_id_no_library = str_replace_all( g_library$CCL, pattern = 
            paste("_", library_name, sep = ""), "" )
        g_library$CCL = cl_id_no_library
        
        if (verbose) {
            message(library_name," amount CCLs: ",
                as.character(nrow(g_library)))
        }
        g_library$Library = rep(library_name, nrow(g_library))
        g_library = g_library[c("CCL","Library","W0","W25","W05","W1")]
        ccls_all <<- rbind(ccls_all, g_library )
    }
    return(ccls_all)
}


#' All variants contained in reference library
#' 
#' This function shows all variants contained in a reference library
#' for a given inclusion weight. Default inclusion weight is 0 
#' (all variants).
#' 
#' @param ref_gen a character vector specifying the reference genome 
#' version. All training sets are associated with a reference genome version. 
#' Default is \code{"GRCH37"}.
#' @param library_name Name of the reference library.
#' @param mutational_weight_inclusion_threshold Include only mutations 
#' with a weight of at least x. Range: 0.0 to 1.0. 1 = unique to CL. 
#' ~0 = found in many CL samples. 
#' @usage
#' show_contained_variants_in_library(
#'     ref_gen,
#'     library_name,
#'     mutational_weight_inclusion_threshold
#' )
#' @import stringr
#' @examples
#' ## Show all variants contained in reference library CELLMINER
#' show_contained_variants_in_library(
#'     ref_gen = "GRCH37",
#'     library_name = "CELLMINER",
#'     mutational_weight_inclusion_threshold = 0
#' )
#' @return Returns a GenomicRanges object that contains the variants
#' @export
show_contained_variants_in_library = function(
    ref_gen = "GRCH37",
    library_name,
    mutational_weight_inclusion_threshold = 0
){
    
    message("Entered reference genome: ", ref_gen, 
        ". Entered library name: ", library_name)
    
    package_path = system.file("", package = "Uniquorn")
    library_path =  paste(
        c(package_path, "/Libraries/", ref_gen), sep = "", collapse= ""
    )
    
    if ( ! dir.exists( library_path ))
        stop("No libraries found!")
    
    libraries = list.dirs(library_path,full.names = FALSE)
    libraries = libraries[ libraries != ""]
    
    if (! (library_name %in% libraries ))
        stop("Could not find library!")
    
    g_mat = read_mutation_grange_objects(
        library_name = library_name,
        ref_gen = ref_gen,
        mutational_weight_inclusion_threshold = 
            mutational_weight_inclusion_threshold,
        test_mode = FALSE
    )
    
    return(g_mat)
}

#' Variants In Cancer Cell Line
#' 
#' This function shows all mutations present in the database 
#' for a selected cancer cell line and reference genome.
#' 
#' @param name_ccl a character vector giving the identifier 
#' of the cancer cell line for which mutations will be shown.
#' @param ref_gen a character vector specifying the reference 
#' genome version. All training sets are associated with a 
#' reference genome version. Default is \code{"GRCH37"}.
#' @param library_name Name of the reference library 
#' @param mutational_weight_inclusion_threshold Include only mutations 
#' with a weight of at least x. Range: 0.0 to 1.0. 1= unique to CL. 
#' ~0 = found in many CCL samples. 
#' @import stringr
#' @usage 
#' show_contained_variants_for_ccl(
#'     name_ccl, 
#'     ref_gen,
#'     library_name,
#'     mutational_weight_inclusion_threshold
#' )
#' @examples 
#' ## Show all mutations for Cancer Cell Line 'SK_OV_3'
#' show_contained_variants_for_ccl(
#'     name_ccl = "SK_OV_3",
#'     ref_gen = "GRCH37",
#'     library_name = "CELLMINER",
#'     mutational_weight_inclusion_threshold = 0
#' )
#' @return GenomicRanges object that contains the ccl's variants
#' @export
show_contained_variants_for_ccl = function(
    name_ccl,
    ref_gen = "GRCH37",
    library_name,
    mutational_weight_inclusion_threshold = 0
){
    
    message("Entered reference genome: ", ref_gen, 
        ". Entered library name: ", library_name,
        ". Entered name of ccl: ", name_ccl)
        
    package_path = system.file("", package = "Uniquorn")
    library_path =  paste(c(package_path, "/Libraries/", ref_gen),
                            sep ="", collapse= "")
    
    if ( ! dir.exists( library_path ))
        stop("No libraries found!")
    
    libraries = list.dirs(library_path,full.names = FALSE)
    libraries = libraries[ libraries != ""]
    
    if (! (library_name %in% libraries ))
        stop("Could not find library!")
    
    g_mat = read_mutation_grange_objects(
        library_name = library_name,
        ref_gen = ref_gen,
        mutational_weight_inclusion_threshold = 
            mutational_weight_inclusion_threshold,
        test_mode = FALSE
    )
    
    g_query_index = stringr::str_detect(g_mat$Member_CCLs, pattern = name_ccl)
    g_query = g_mat[g_query_index,]
    
    return(g_query)
}

#' Cancer cell lines with specific variant
#' 
#' This function displays all cancer cell lines in the database which 
#' contain a specified variant. Utilizes closed interval coordinates.
#' 
#' @param start Start coordinate
#' @param end Stop coordinate
#' @param chromosome Chromosome, 'chr' prefixes are ignored
#' @param ref_gen a character vector specifying the reference genome 
#' version. All training sets are associated with a reference genome 
#' version. Default is \code{"GRCH37"}.
#' @param library_name Name of the reference library 
#' @param mutational_weight_inclusion_threshold Include only mutations 
#' with a weight of at least x. Range: 0.0 to 1.0. 1= unique to CL. 
#' ~0 = found in many CCL samples. 
#' @import stringr
#' @importFrom IRanges IRanges
#' @importFrom IRanges subsetByOverlaps
#' @usage 
#' show_which_ccls_contain_variant(
#'     start,
#'     end,
#'     chromosome,
#'     ref_gen,
#'     library_name,
#'     mutational_weight_inclusion_threshold
#' )
#' @examples
#' show_which_ccls_contain_variant(
#'     start = 92030762,
#'     end = 92030762,
#'     chromosome = 8,
#'     ref_gen = "GRCH37",
#'     library_name = "CELLMINER",
#'     mutational_weight_inclusion_threshold = 0
#' )
#' @return Returns a GenomicRanges object that contains the variant if present.
#' Member ccls can be found in the $Member_ccl vector
#' @export
show_which_ccls_contain_variant = function(
    start,
    end,
    chromosome,
    ref_gen = "GRCH37",
    library_name,
    mutational_weight_inclusion_threshold = 0
){
    
    message("Entered reference genome: ", ref_gen, 
        ". Entered library name: ", library_name)
    
    package_path = system.file("", package = "Uniquorn")
    library_path = paste(
        c(package_path, "/Libraries/", ref_gen), sep ="", collapse= ""
    )
    
    if ( ! dir.exists( library_path ))
        stop("No libraries found!")
    
    libraries = list.dirs(library_path,full.names = FALSE)
    libraries = libraries[ libraries != ""]
    
    if (! (library_name %in% libraries ))
        stop("Could not find library!")
    
    g_mat = read_mutation_grange_objects(
        library_name = library_name,
        ref_gen = ref_gen,
        mutational_weight_inclusion_threshold = 
        mutational_weight_inclusion_threshold,
        test_mode = FALSE
    )
    
    chromosome = stringr::str_to_upper(chromosome)
    chroms = stringr::str_replace( chromosome, pattern = "^CHR" , "")
    
    g_query = GenomicRanges::GRanges(
        seqnames = c( chroms ),
        IRanges::IRanges(
            start = as.integer( c(start) ),
            end = as.integer( c(end) )
        )
    )
    
    g_query = IRanges::subsetByOverlaps(g_mat, g_query)
    return(g_query)
}

#' Library Name Reader
#' 
#' This function procides information on the reference library names
#' 
#' @param ref_gen a character vector specifying the reference genome 
#' version. All training sets are associated with a reference genome 
#' version. Default is \code{"GRCH37"}.
#' @import stringr
#' @usage
#' read_library_names(ref_gen)
#' @examples
#' read_library_names(ref_gen = "GRCH37")
#' @return Returns a character vector of the contained libraries
#' @export
read_library_names = function(
    ref_gen
){
    
    package_path = system.file("", package = "Uniquorn")
    library_path =  paste(c(package_path, "/Libraries/", ref_gen, "/"), 
        sep ="", collapse= "")
    
    library_names = list.dirs(library_path, full.names = FALSE)
    library_names = library_names[library_names!= ""]
    
    return(library_names)
}

context("Correct add and remove of custom Cancer Cell Line")

test_that("adding and removing a custom Cancer Cell Line works succesfully", {

    # Check add
    HT29_vcf_file = system.file("extdata/HT29_TEST.vcf", package = "Uniquorn")
    expect_message(
        add_custom_vcf_to_database(
            vcf_input_files = HT29_vcf_file,
            ref_gen = "GRCH37",
            library_name = "CELLMINER",
            n_threads = 1,
            test_mode = TRUE
        ),
        "Finished"
    )
  
    #ccls_all = show_contained_ccls()
    #variants = show_contained_variants_for_ccl(name_ccl = "HT29_TEST", library_name = "CELLMINER")
  
    #expect_equal(dim(ccls_all)[1], ccls_before + 1)
    #expect_true(any(grepl("HT29_TEST", ccls_all[,1], fixed = TRUE)))
    #expect_equal(length(variants), 24969)
  
    # Check remove
    expect_message(
        remove_ccls_from_database(
            ccl_names = "786_0",
            ref_gen = "GRCH37",
            library_name = "CELLMINER",
            test_mode = TRUE
        ),
        "Finished removing all cancer cell lines"
    )
    #ccls_all = show_contained_ccls()
    #variants = show_contained_variants_for_ccl(name_ccl = "HT29_TEST", library_name = "CELLMINER")
  
    #expect_equal(dim(ccls_all)[1], ccls_before)
    #expect_false(any(grepl("HT29_TEST", ccls_all[,1], fixed = TRUE)))
    #expect_equal(length(variants), 0)
})

context("Correct identification results for test data")

test_that("identification of test data returns correct results", {
    HT29_vcf_file = system.file("extdata", "HT29.vcf", package = "Uniquorn")
    ident_result = identify_vcf_file(
        HT29_vcf_file, 
        ref_gen = "GRCH37", 
        verbose = FALSE,
        write_results = FALSE
    )
    
    libraries = read_library_names(ref_gen = "GRCH37")
    ccls_all = show_contained_ccls()
    ccls = ccls_all[,1]
    ccls = str_replace_all(ccls, pattern = 
        paste(c("\\_","\\-","\\(","\\)"), collapse = "|", sep = ""), "")
    ccls = str_to_upper(ccls)
    occ = sum(ccls %in% "HT29")
    index = which(ccls %in% "HT29")
    names = ccls_all[index, 1]
    
    expect_is(ident_result, "data.frame")
    expect_equal(dim(ident_result)[2], 7)
  
    index = which(ident_result[,1] %in% names)
    expect_equal(length(index), occ)
    expect_true(all(ident_result$P_value_sig[index]))
    expect_true(all(ident_result$Above_Penality[index]))
    expect_true(all(ident_result$Identification_sig[index]))
    expect_equal(sort(unique(as.character(ident_result$Library))), sort(libraries))

})

context("Correct access and structure of stored database information")
names = read_library_names(ref_gen = "GRCH37")

test_that("show_contained_ccls returns correct object", {
    names = read_library_names(ref_gen = "GRCH37")
    ccls_all = show_contained_ccls()
  
    expect_is(ccls_all, "data.frame")
    expect_equal(dim(ccls_all)[2], 6)
    expect_equal(sort(unique(ccls_all$Library)), sort(names))
    expect_is(ccls_all[,1], "character")
    expect_is(ccls_all[,2], "character")

})

test_that("show_contained_variants_in_library returns correct object", {
    names = read_library_names(ref_gen = "GRCH37")
    if("CELLMINER" %in% names){
        
        g_ranges = show_contained_variants_in_library(
            library_name = "CELLMINER"
        )
        
        expect_is(g_ranges, "GRanges")
        expect_true(length(g_ranges) > 50000)
        expect_equal(sum(grepl("CELLMINER", g_ranges$Member_CCLs)), length(g_ranges))
    }
})

test_that("show_contained_variants_for_ccl returns the right object", {
    names = read_library_names(ref_gen = "GRCH37")
    ccls_all = show_contained_ccls()
    if("CELLMINER" %in% names){
      
        index = ccls_all$Library == "CELLMINER"
        ccls_all = ccls_all[index,]
        
        ccl_index = sample(1:length(ccls_all), 1, replace = FALSE)
        ccl = ccls_all$CCL[ccl_index]
        
        variants = show_contained_variants_for_ccl(
            name_ccl = ccl, library_name = "CELLMINER"
        )
  
        expect_is(variants, "GRanges")
        expect_true(length(variants) > 1)
        
        ccl_index = sample(1:length(ccls_all), 1, replace = FALSE)
        ccl = ccls_all$CCL[ccl_index]
        
        variants = show_contained_variants_for_ccl(
          name_ccl = ccl, library_name = "CELLMINER"
        )
        
        expect_is(variants, "GRanges")
        expect_true(length(variants) > 1)
  }
})

#' Remove Cancer Cell Line
#'
#' This function removes a cancer cell line training fingerprint (VCF file)
#'  from the database. The names of all training sets can 
#'  be seen by using the function \code{show_contained_cls}.
#' 
#' @param ccl_names A character vector giving the names of the cancer cell line
#'  identifiers to be removed. Can be one or many
#' @param ref_gen A character vector specifying the reference genome version.
#'  All training sets are associated with a reference genome version. 
#'  Default is \code{"GRCH37"}.
#' @param library_name Name of the library from which ccls are to be removed
#' @param test_mode Signifies if this is a test run
#' @import GenomicRanges stringr
#' @usage 
#' remove_ccls_from_database(ccl_names, ref_gen = "GRCH37",
#'     library_name, test_mode = FALSE)
#' @examples 
#' remove_ccls_from_database(
#'     ccl_names = "HT29",
#'     ref_gen = "GRCH37",
#'     library_name = "CELLMINER",
#'     test_mode = TRUE
#' )
#' @return Message that indicates whether the removal was succesful.
#' @export
remove_ccls_from_database = function( 
  ccl_names,
  ref_gen = "GRCH37",
  library_name,
  test_mode = FALSE
){
  message("Reference genome: ", ref_gen)
  
  package_path = system.file("", package = "Uniquorn")
  library_path = paste(c(package_path, "/Libraries/", ref_gen),
                       sep ="", collapse= "")
  
  if (length(library_name) > 1)
    stop("Cannot process more than one library per run, please 
         provide single libraries.")
  
  g_mat = read_mutation_grange_objects(
    library_name = library_name,
    ref_gen = ref_gen,
    mutational_weight_inclusion_threshold = 0
  )
  
  for (ccl_id in ccl_names){
    
    ccl_lib = paste(ccl_id, library_name, sep = "_")
    message("Deleting ", ccl_id)
    search_term = paste(c(
      paste( "(",ccl_lib,",",")", sep = "" ),
      paste( "(",ccl_lib,")", sep = "" ),
      "(.*,$)"
    ), collapse= "|")
    
    member_ccls = str_replace_all(mcols(g_mat)$Member_CCLs,
                                  pattern = search_term, replacement = "")
    member_ccls = str_replace_all(member_ccls, pattern = ",$", "")
    
    non_empty_vec = member_ccls != ""
    g_mat = g_mat[non_empty_vec,]
    member_ccls = member_ccls[ non_empty_vec ]
    mcols(g_mat)$Member_CCLs = as.character(member_ccls)
    
    message(
      "Excluded ",
      as.character(sum(non_empty_vec == FALSE)),
      " variant entries after removal."
    )
    
    if( test_mode == FALSE ){
      stats_path = paste( c( library_path,"/",library_name,
                             "/CCL_List_Uniquorn_DB.rds"), sep ="", collapse= "")
      g_library = readRDS(stats_path)
      g_library = data.frame(g_library, stringsAsFactors = FALSE)
      g_library = g_library[ g_library$CCL!= "",]
      g_library = g_library[!is.na(g_library$CCL),]
      g_library = g_library[g_library$CCL != ccl_id,]
      saveRDS(g_library, file = stats_path)
    }
  }
  
  #member_ccls = str_replace(member_ccls, pattern = ",$","")
  message("Finished removing all ccls. Recalculating DB")
  
  if( test_mode == FALSE){
    write_w0_and_split_w0_into_lower_weights(
      g_mat = g_mat,
      ref_gen = ref_gen,
      library_name = library_name
    )
  }
  message("Finished removing all cancer cell lines")
}


#' Remove entire Library from Database
#'
#' This function removes a entire library from the database by removing all 
#' associated cancer cell line fingerprints from the database.
#' 
#' @param library a character vector giving the names of the library to be
#'  removed.
#' @param ref_gen a character vector specifying the reference genome version.
#'  All training sets are associated with a reference genome version. 
#'  Default is \code{"GRCH37"}.
#' @param test_mode is this a test? Just for internal use.
#' @import stringr
#' @usage 
#' remove_library_from_database(library, ref_gen = "GRCH37", test_mode = FALSE)
#' @examples 
#' remove_library_from_database(library = "CELLMINER",
#'                                 ref_gen = "GRCH37",
#'                                 test_mode = TRUE)
#' @return Message that indicates whether the removal was succesful.
#' @export
remove_library_from_database = function( 
    library,
    ref_gen = "GRCH37",
    test_mode = FALSE
){
    message("Reference genome: ", ref_gen)
    message("Removing library ", library, " and all associated",
            " cancer cell lines.")
    package_path = system.file("", package = "Uniquorn")
    present = list.dirs(path = paste(package_path, "Libraries", ref_gen, sep = "/"), 
                        recursive = FALSE, full.names = FALSE)
    if(!(library %in% present)){
        stop("Specified library is not present in Uniquorn database.")
    }
    library_path = paste(package_path, "Libraries", ref_gen, library,
                         sep ="/")
    unlink(library_path, recursive = !test_mode)
    
    remain = list.dirs(path = paste(package_path, "Libraries", ref_gen, sep = "/"), 
                       recursive = FALSE, full.names = FALSE)
    if(library %in% remain){
        message("Removal not successful.")
    } else{
        message("Removal successful.")
    }
}

#' add_custom_vcf_to_database
#
#' This function adds the variants of parsed custom CCLs to a monet DB instance
#'  
#' @param vcf_input_files a character vector containing the input vcf files.
#'  This may be one or many vcf files.
#' @param ref_gen a character string specifying the reference genome version.
#'  All training sets are associated with a reference genome version.
#'  Default is \code{"GRCH37"}. 
#' @param library_name a character string giving the name of the library to add the
#'  cancer cell lines to. Default is \code{"CUSTOM"}. 
#'  Library name will be automatically added as a suffix to the identifier.
#' @param n_threads an integer specifying the number of threads to be used.
#' @param test_mode Is this a test? Just for internal use
#' @return Message wheather the adding was successful
#' @import GenomicRanges foreach
#' @usage 
#' add_custom_vcf_to_database(
#'     vcf_input_files,
#'     ref_gen = "GRCH37",
#'     library_name = "CUSTOM",
#'     n_threads = 1,
#'     test_mode = FALSE
#' )
#' @examples
#' HT29_vcf_file = system.file("extdata/HT29_TEST.vcf", package = "Uniquorn");
#' add_custom_vcf_to_database(
#'     vcf_input_files = HT29_vcf_file,
#'     library_name = "CELLMINER",
#'     ref_gen = "GRCH37",
#'     n_threads = 1,
#'     test_mode = TRUE
#' )
#' 
#' @export
add_custom_vcf_to_database = function(
    vcf_input_files,
    ref_gen = "GRCH37",
    library_name = "CUSTOM",
    n_threads = 1,
    test_mode = FALSE)
{
    
    #ccl_list = read_ccl_list(ref_gen = ref_gen, library_name = library_name)
    
    if (n_threads > 1){
        
        doParallel::registerDoParallel(n_threads)
        
        foreach::foreach(
            vcf_input_file = vcf_input_files
        ) %dopar% {
            g_query = parse_vcf_file(
                vcf_input_file,
                ref_gen = ref_gen,
                library_name = library_name
            )
            
            parse_vcf_query_into_db(
                g_query,
                ref_gen = ref_gen,
                test_mode = test_mode,
                library_name = library_name
            )
        }
        
        doParallel::stopImplicitCluster()
        
    } else {
        
        for (vcf_input_file in vcf_input_files){
            
            g_query = parse_vcf_file(
                vcf_input_file,
                ref_gen = ref_gen,
                library_name = library_name
            )
            
            parse_vcf_query_into_db(
                g_query,
                ref_gen = ref_gen,
                test_mode = test_mode,
                library_name = library_name
            )
        }
    }
    message("Finished")
}

#' parse_vcf_query_into_db
#
#' This function adds the variants of parsed custom CCLs to a monet DB instance
#'  
#' @param g_query a GenomicRanges object
#' @param ref_gen a character string specifying the reference genome version.
#'  All training sets are associated with a reference genome version.
#'  Default is \code{"GRCH37"}. 
#' @param library_name a character string giving the name of the library to add
#'  the cancer cell lines to. Default is \code{"CUSTOM"}. 
#'  Library name will be automatically added as a suffix to the identifier.
#' @param test_mode Is this a test? Just for internal use
#' @return Message wheather the adding was successful
#' @import GenomicRanges stringr
#' @importFrom IRanges findOverlaps
parse_vcf_query_into_db = function(
    g_query,
    ref_gen = "GRCH37",
    library_name,
    test_mode = FALSE
){
    
    cl_id = mcols( g_query )$Member_CCLs[1]
    cl_id = str_replace( cl_id, pattern = paste0("_", library_name), "")
    
    cl_data =  show_contained_ccls(verbose = FALSE)
    if (cl_id %in% cl_data$CCL[ cl_data$Library == library_name  ] ){
        stop("CCL ", cl_id, " already contained in DB ", library_name,
            ". Remove first or change name")
    }
    
    message("Sample: ", cl_id,", Library: ", library_name)
    
    g_mat = read_mutation_grange_objects(
        library_name = library_name,
        ref_gen = ref_gen,
        mutational_weight_inclusion_threshold = 0,
        test_mode = test_mode
    )
    
    if ( "Member_CCLs" %in% names(mcols(g_mat)) ){
        
        g_mat_new = unique(c(
            GenomicRanges::GRanges(g_mat),
            GenomicRanges::GRanges(g_query)
        ))
        fo_g_mat = findOverlaps(
            query = g_mat,
            subject = g_mat_new,
            select = "arbitrary",
            type = "equal"
        )
        
    } else {
        
        g_mat_new = unique(GenomicRanges::GRanges(g_query))
        fo_g_mat = GenomicRanges::GRangesList()
    }
    GenomicRanges::mcols(g_mat_new)$Member_CCLs = 
        rep("", length(g_mat_new))
    
    fo_query = findOverlaps(
        query = g_query,
        subject = g_mat_new,
        select = "arbitrary",
        type = "equal"
    )
    
    old_members = as.character( 
        sapply(as.character(elementMetadata(g_query)$Member_CCLs), 
            function(vec){
                ccl_ids = as.character(unlist(str_split(vec, pattern = ",")))
                ccl_ids = unique(ccl_ids)
                ccl_ids = paste(ccl_ids, collapse = ",", sep ="")
                return(ccl_ids)
            }
        )
    )
    
    mcols( g_mat_new )$Member_CCLs[fo_query] = old_members
    
    if ( "Member_CCLs" %in% names(mcols(g_mat)) )
        
        GenomicRanges::mcols( g_mat_new )$Member_CCLs[fo_g_mat]   =
            paste( 
                mcols( g_mat_new )$Member_CCLs[fo_g_mat],
                elementMetadata(g_mat)$Member_CCLs, sep = ","
            )
    
    GenomicRanges::mcols( g_mat_new )$Member_CCLs = stringr::str_replace( 
        GenomicRanges::mcols( g_mat_new )$Member_CCLs,
        pattern = "^,",""
    )
    g_mat = g_mat_new
    
    if(test_mode == FALSE){
        write_w0_and_split_w0_into_lower_weights(
            g_mat = g_mat,
            ref_gen = ref_gen,
            library_name = library_name
        )
    }
    
    message("Finished parsing ", cl_id, ", library: ", library_name)
}

#' identify_VCF_file
#' 
#' Identifies a cancer cell lines contained in a vcf file based 
#' on the pattern (start & length) of all contained mutations/ variations.
#' 
#' \code{identify_vcf_file} parses the vcf file and predicts 
#' the identity of the sample
#' 
#' @param write_xls Create identification results additionally 
#' as xls file for easier reading
#' @param vcf_file Input vcf file. Only one sample column allowed.
#' @param output_file Path of the output file. If blank, 
#' autogenerated as name of input file plus '_uniquorn_ident.tab' suffix.
#' @param ref_gen Reference genome version. All training sets are 
#' associated with a reference genome version. Default: GRCH37
#' @param mutational_weight_inclusion_threshold Include only mutations 
#' with a weight of at least x. Range: 0.0 to 1.0. 1= unique to CL. 
#' ~0 = found in many CL samples. 
#' @param minimum_matching_mutations The minimum amount of mutations that 
#' has to match between query and training sample for a positive prediction
#' @param top_hits_per_library Limit the number of significant similarities
#' per library to n (default 3) many hits. Is particularrly used in contexts
#' when heterogeneous query and reference CCLs are being compared.
#' @param manual_identifier Manually enter a vector of CL 
#' name(s) whose bed files should be created, independently from 
#' them passing the detection threshold
#' @param output_bed_file If BED files for IGV visualization should be 
#' created for the Cancer Cell lines that pass the threshold
#' @param verbose Print additional information
#' @param p_value Required p-value for identification.
#' Note that if you set the confidence score, the confidence score
#' overrides the p-value
#' @param confidence_score Cutoff for positive prediction between 0 and 100.
#' Calculated by transforming the p-value by -1 * log(p-value)
#' Note that if you set the confidence score, the confidence score
#' overrides the p-value
#' @param n_threads Number of threads to be used
#' @param write_results Write identification results to file
#' @import WriteXLS
#' @usage 
#' identify_vcf_file( 
#'     vcf_file,
#'     output_file,
#'     ref_gen,
#'     minimum_matching_mutations,
#'     mutational_weight_inclusion_threshold,
#'     write_xls,
#'     output_bed_file,
#'     top_hits_per_library,
#'     manual_identifier,
#'     verbose,
#'     p_value,
#'     confidence_score,
#'     n_threads,
#'     write_results
#' )
#' @examples 
#' HT29_vcf_file = system.file("extdata/HT29.vcf", package = "Uniquorn");
#' 
#' identification = identify_vcf_file(
#'     vcf_file = HT29_vcf_file, 
#'     verbose = FALSE,
#'     write_results = FALSE
#' )
#' @return R table with a statistic of the identification result
#' @export
identify_vcf_file = function(
    vcf_file,
    output_file = "",
    ref_gen = "GRCH37",
    minimum_matching_mutations = 0,
    mutational_weight_inclusion_threshold = 0.5,
    write_xls = FALSE,
    output_bed_file = FALSE,
    top_hits_per_library = 3,
    manual_identifier = "",
    verbose = TRUE,
    p_value = .05,
    confidence_score = NA,
    n_threads = 1,
    write_results = FALSE
){
    
    if ( ! is.na(confidence_score) ){
        message(
            "Confidence score has been set to ",
            as.character(confidence_score),
            ", overriding the p_value"
        )
        confidence_score[confidence_score < 0 ] = 0
        confidence_score[confidence_score > 100 ] = 100
        p_value = exp(-1 * confidence_score)
    }
    
    g_query = parse_vcf_file(
        vcf_file,
        ref_gen = ref_gen,
        library_name = ""
    )
    
    library_names = read_library_names(ref_gen = ref_gen)
    match_t <<- data.frame(
        CCL = as.character(),
        Matches = as.character(),
        Library = as.character()
    )
    
    message(
        "Limiting out to top ",
        as.character(top_hits_per_library),
        " hits per library."
    )
    
    for( library_name in library_names ){
        
        options(warn = -1)
        hit_list = match_query_ccl_to_database(
            g_query,
            ref_gen = ref_gen,
            library_name = library_name,
            mutational_weight_inclusion_threshold =
                mutational_weight_inclusion_threshold
        )
        options(warn = 0)
        #assign("match_t", rbind(match_t,hit_list),envir = parent.frame())
        match_t = rbind(match_t, hit_list)
        
        message(
            library_name, ": ",
            as.character(hit_list$CCL[1]),
            ", matching variants: ",
            as.character(hit_list$Matches[1])
        )
    }
    
    # statistics
    
    match_t = add_p_q_values_statistics(
        g_query,
        match_t,
        p_value,
        ref_gen = ref_gen,
        minimum_matching_mutations = minimum_matching_mutations,
        top_hits_per_library
    )
    match_t = add_penalty_statistics(
      match_t,
      minimum_matching_mutations
    )
    match_t$Identification_sig = match_t$P_value_sig & match_t$Above_Penalty
    match_t = match_t[order(as.double(match_t$P_values),decreasing = FALSE),]
    
    match_t$Matches = as.integer(match_t$Matches)
    match_t$All_variants = as.integer(match_t$All_variants)
    
    ### io stuff
    
    if(output_file == ""){
        output_file = str_replace(
            vcf_file,pattern = "(\\.vcf)|(\\.VCF)", ".ident.tsv" )
    }
    
    if (verbose){
        
        message("Candidate(s): ", paste0(unique( 
            as.character( match_t$CCL )[ match_t$Identification_sig  ]))
        )
        message("Storing information in table: ", output_file)
    }
       
    if (write_results){ 
        utils::write.table( 
            match_t,
            output_file,
            sep ="\t",
            row.names = FALSE,
            quote = FALSE
        )
    }
    
    if (output_bed_file & ( sum( as.logical(match_t$Q_value_sig) ) > 0 ))
        create_bed_file( 
            match_t, 
            vcf_fingerprint, 
            output_file, 
            manual_identifier
        )
    
    if ( !verbose )
        match_t = match_t[ ,
            !( colnames( match_t ) %in% c(
                "P_values",
                "Q_values",
                "Q_value_sig"
                )
            )
        ]

    
    if ( write_xls )

        WriteXLS::WriteXLS( 
            x = match_t,
            path.expand(
                output_file_xls
            ),
            row.names = FALSE
        )
    
    return( match_t )
}

#' match_query_ccl_to_database
#' 
#' Matches query ccl to the database
#' 
#' @param g_query IRanges object that contains the variants
#' @param ref_gen Reference genome version. All training sets are 
#'  associated with a reference genome version. Default: GRCH37
#' @param library_name a character string giving the name of the library
#' @param mutational_weight_inclusion_threshold a numerical giving
#'  the lower bound for mutational weight to be included
#' @importFrom IRanges subsetByOverlaps
#' @return The R Table sim_list which contains the CoSMIC CLP fingerprints 
match_query_ccl_to_database = function(
    g_query,
    ref_gen = "GRCH37",
    library_name,
    mutational_weight_inclusion_threshold
){
    
    g_mat = read_mutation_grange_objects(
        ref_gen = ref_gen,
        library_name = library_name,
        mutational_weight_inclusion_threshold = 
            mutational_weight_inclusion_threshold,
        test_mode = FALSE
    )
    
    fo_query = GenomicRanges::findOverlaps(
        query = g_query,
        subject = g_mat,
        select = "arbitrary",
        type = "equal"
    )
    match = IRanges::subsetByOverlaps(g_mat, g_query)
    cl_ids = as.character(unlist(
        str_split(GenomicRanges::mcols(match)$Member_CCLs, pattern = "," )
    ))
    cl_ids = str_replace_all(cl_ids,
                            pattern = paste("_",library_name,sep = ""),"" )
    
    matching_variants = sort(table(cl_ids), decreasing = TRUE)
    cl_ids_all = str_replace_all( 
        GenomicRanges::mcols(g_mat)$Member_CCLs, 
        pattern = paste("_",library_name,sep = ""),
        ""
    )
    cl_ids_all = unique(as.character(
        unlist(str_split(cl_ids_all,pattern = ","))))
    missing_ccls = table(
        cl_ids_all[ !( cl_ids_all %in% names(matching_variants))]
    )
    missing_ccls = missing_ccls - 1
    matching_variants = c(matching_variants, missing_ccls)
    
    matching_variants_t = data.frame(
        CCL = names(matching_variants),
        Matches = as.character(matching_variants),
        Library = rep(library_name, length(matching_variants))
    )
    
    ### add stats info
    
    cl_data <<-  show_contained_ccls( verbose = FALSE, ref_gen = ref_gen)
    cl_data = cl_data[cl_data$Library == library_name,]
    
    switch(as.character(mutational_weight_inclusion_threshold),
        "0" = { weight_name = "W0"  },
        "0.25" = { weight_name = "W25"  },
        "0.5" = { weight_name = "W05"  },
        "1" = { weight_name = "W1"  },
        stop(paste( "Could not recocgnize mutational weight ", 
            mutational_weight_inclusion_threshold))
    )
    
    all_muts = as.character(cl_data[[weight_name]])
    ccl_match = match( as.character(
        matching_variants_t$CCL), as.character(cl_data$CCL), nomatch = 0)
    
    missing_libraries = unique(
        as.character(matching_variants_t$Library[ccl_match == 0]))
    
    if(length(missing_libraries) > 0){
        
        for (mis_lib_name in  missing_libraries){
            
            message("Updating database for library: ", mis_lib_name)
            mis_mat = read_mutation_grange_objects(
                ref_gen = ref_gen,
                library_name = mis_lib_name,
                mutational_weight_inclusion_threshold = 
                    mutational_weight_inclusion_threshold
            )
            write_w0_and_split_w0_into_lower_weights(
                g_mat = mis_mat,
                library_name = mis_lib_name,
                ref_gen = ref_gen
            )
        }
        
        cl_data <<-  show_contained_ccls( ref_gen = ref_gen, verbose = FALSE)
        cl_data = cl_data[cl_data$Library == library_name,]
        
        switch(as.character(mutational_weight_inclusion_threshold),
            "0" = { weight_name = "W0"  },
            "0.25" = { weight_name = "W25"  },
            "0.5" = { weight_name = "W05"  },
            "1" = { weight_name = "W1"  },
            stop("Could not recocgnize mutational weight ", 
                mutational_weight_inclusion_threshold)
        )
        
        all_muts <<- as.character(cl_data[[weight_name]])
        ccl_match <<- match( as.character(
            matching_variants_t$CCL
        ), as.character(cl_data$CCL), nomatch = 0)
    }
    
    matching_variants_t$All_variants = all_muts[ccl_match]
    
    return(matching_variants_t)
}

write_mutation_grange_objects = function(
    g_mat,
    library_name,
    mutational_weight_inclusion_threshold,
    ref_gen,
    type = ""
){
    
    package_path = system.file("", package = "Uniquorn")
    
    rdata_path =  paste( c(
        package_path,"/Libraries/",
        ref_gen,"/",library_name,
        "/W",as.character(mutational_weight_inclusion_threshold),
        "_Uniquorn_DB.rds"),
        sep ="", collapse= ""
    )
    
    if ( type != ""){
        rdata_path = str_replace(rdata_path, pattern = "_Uniquorn_DB.rds",
            paste( c( ".",type,"_Uniquorn_DB.rds"), sep = "",collapse = "" )
        )
    }
    
    dir.create(
        paste( 
            c(package_path,"/Libraries/",ref_gen,"/"),
            collapse = "",
            sep =""
        ),
        showWarnings = FALSE)
    dir.create(
        paste( 
            c(package_path,"/Libraries/",ref_gen,"/",library_name),
            collapse = "",
            sep =""
        ),
        showWarnings = FALSE
    )
    saveRDS(g_mat,rdata_path)
}

#' read_mutation_grange_objects
#' 
#' Read the GRange object for a specific library
#' 
#' @param library_name a character string giving the name of the library
#' @param mutational_weight_inclusion_threshold a numerical giving the
#'  lower bound for mutational weight to be included
#' @param ref_gen Reference genome version. All training sets are 
#'  associated with a reference genome version. Default: GRCH37
#' @param test_mode Is this a test? Just for internal use
#' @importFrom IRanges IRanges
#' @return The R Table sim_list which contains the CoSMIC CLP fingerprints 
read_mutation_grange_objects = function(
    library_name,
    mutational_weight_inclusion_threshold,
    ref_gen,
    test_mode
){
    
    package_path = system.file("", package = "Uniquorn")
    
    rdata_path =  paste( c(
        package_path,"/Libraries/",
        ref_gen,"/",library_name,
        "/W",as.character(mutational_weight_inclusion_threshold),
        "_Uniquorn_DB.rds"),
        sep = "",
        collapse = ""
    )
    
    if (! file.exists( rdata_path )){
        message(paste(c("Database not found: ", rdata_path,
                        ", creating new database."),collapse = "",sep =""))
        g_mat = GenomicRanges::GRanges(
            seqnames = NULL,
            IRanges(
                start = NULL,
                end = NULL
            )
        )
        
        if (test_mode == FALSE){
            dir.create(
                paste( 
                    c(package_path,"/Libraries/"),
                    collapse = "",
                    sep = ""
                ),
                showWarnings = FALSE
            )
            dir.create(
                paste( 
                    c(package_path,"/Libraries/",ref_gen,"/"),
                    collapse = "",
                    sep = ""
                ),
                showWarnings = FALSE
            )
            dir.create(
                paste( 
                    c(package_path,"/Libraries/",ref_gen,"/",library_name),
                    collapse = "",
                    sep = ""
                ),
                showWarnings = FALSE
            )
        }
    } else {
        g_mat = readRDS(rdata_path)
    }
    
    return(g_mat)
}

#' initiate_canonical_databases
#' 
#' Parses data into r list variable
#' 
#' @param cosmic_file The path to the Cosmic CLP file. The Cosmic file 
#' can be obtained from "https://cancer.sanger.ac.uk/cell_lines/download" and 
#' should be labeled "CosmicCLP_MutantExport.tsv.gz".
#' Ensure that the right reference genome is used
#' @param ccle_file The path to the ccle DNA genotype data file. 
#' It should be labeled "CCLE_mutations.csv".
#' Ensure that the right reference genome is used
#' @param ccle_sample_file The path to the CCLE sample file.
#' It should be labeled "sample_info.csv" containing both the
#' DepMap ID and corresponding cell line name.
#' @param ref_gen Reference genome version
#' @return Returns message if parsing process has succeeded
#' @import R.utils stringr
#' @usage
#' initiate_canonical_databases(
#'     cosmic_file = "CosmicCLP_MutantExport.tsv.gz",
#'     ccle_file = "CCLE_mutations.csv",
#'     ccle_sample_file = "sample_info.csv",
#'     ref_gen = "GRCH38"
#' )
#' @examples 
#' initiate_canonical_databases(
#'     cosmic_file = "CosmicCLP_MutantExport.tsv.gz",
#'     ccle_file = "CCLE_mutations.csv",
#'     ccle_sample_file = "sample_info.csv",
#'     ref_gen = "GRCH38"
#' )
#' @export
initiate_canonical_databases = function(
    cosmic_file = "CosmicCLP_MutantExport.tsv.gz",
    ccle_file = "CCLE_mutations.csv",
    ccle_sample_file = "sample_info.csv",
    ref_gen = "GRCH38"
){

    message("Reference genome: ", ref_gen)

    # Parse CoSMIC file
    if (file.exists(cosmic_file)){
        message("Found CoSMIC: ", file.exists(cosmic_file))
        
        if (grepl(".gz$", cosmic_file, ignore.case = TRUE)){
            gunzip(cosmic_file, overwrite = TRUE)
            cosmic_file = gsub(".gz$", "", cosmic_file, ignore.case = TRUE)
        }
        parse_cosmic_genotype_data( cosmic_file, ref_gen = ref_gen )
    }

    # Parse CCLE file
    if (file.exists(ccle_file) && file.exists(ccle_sample_file)){
        message("Found CCLE and sample info: ", 
                file.exists(ccle_file) && file.exists(ccle_sample_file))
        parse_ccle_genotype_data(ccle_file, ccle_sample_file, ref_gen = ref_gen)
    }
    
    if ((!file.exists(cosmic_file)) & (!file.exists(ccle_file))){ 
        warning("Did neither find CCLE & CoSMIC file! Aborting.")
    } else {
        message("Finished parsing, ",
            "aggregating over parsed Cancer Cell Line data")
        
        message("Finished aggregating, saving to database")

        message("Initialization of Uniquorn DB finished")
    }
}

#' parse_cosmic_genotype_data
#' 
#' Parses cosmic genotype data
#' 
#' @param cosmic_file Path to cosmic clp file in hard disk
#' @param ref_gen Reference genome version
#' @importFrom stats aggregate
#' @importFrom data.table fread
#' @import IRanges
#' @return The R Table sim_list which contains the CoSMIC CLP fingerprints 
#' @keywords internal
parse_cosmic_genotype_data = function(cosmic_file, ref_gen = "GRCH38"){
    
    # Only read in columns specified with subset
    library_name = "COSMIC"
    package_path = system.file("", package = "Uniquorn")
    
    subset = c(5, 26)
    
    if (!grepl("CosmicCLP_MutantExport", cosmic_file)){ 
        stop("Warning. This is not the recommended COSMIC genotype file!",
            " The recommended file is the 'CosmicCLP_MutantExport.tsv.gz'",
            " file.")
        subset = c(5, 19)
    }
    
    cosmic_genotype_tab = data.table::fread(cosmic_file, select = subset,
       sep = "\t", showProgress = FALSE)
    colnames(cosmic_genotype_tab) = c("sample", "position") 
    
    # Remove CLs with NA or "" position
    empty_position <- c(which(cosmic_genotype_tab$position == ""), 
                        which(is.na(cosmic_genotype_tab$position)))
    cosmic_genotype_tab <- cosmic_genotype_tab[-empty_position,]
    
    # Extract and process coordinates and CL IDs
    message("Parsing Cosmic Coordinates")
    coords = cosmic_genotype_tab[, gsub(":|-", "_", position)]
    seq_name = vapply(strsplit(coords, "_"), `[`, 1, FUN.VALUE = character(1))
    starts = vapply(strsplit(coords, "_"), `[`, 2, FUN.VALUE = character(1))
    ends = vapply(strsplit(coords, "_"), `[`, 3, FUN.VALUE = character(1))

    cls = cosmic_genotype_tab[
        ,gsub("/|(|])| ", "", sample, ignore.case = TRUE)]
    cls[cls == "KM-H2"] = "KMH2"
    cls[cls == "KMH-2"] = "KMH2ALTERNATIVE"
    
    c_matches = match(coords, unique(coords), nomatch = 0)
    
    message("Aggregating Cosmic CCL names")
    new_cls = data.table::data.table(
      "CLS" = cls,
      "Index" = c_matches
    )
    new_cls = new_cls[,lapply(.SD, paste, sep = "", collapse= ","), by = Index]
    
    # Extract and process coordinates and CL IDs
    g_mat = GenomicRanges::GRanges(
        seqnames = seq_name,
        IRanges(
            start = as.integer( starts ),
            end = as.integer( ends )
        )
    )
    g_mat = unique(g_mat)
    mcols(g_mat)$Member_CCLs = new_cls$CLS
    mcols(g_mat)$Member_CCLs = str_replace_all(mcols(g_mat)$Member_CCLs, 
        pattern = paste( "_", library_name, sep =""), "")
    
    message("Normalizing CCL names")
    
    write_mutation_grange_objects(
        g_mat = g_mat,
        library_name = library_name,
        ref_gen = ref_gen, 
        mutational_weight_inclusion_threshold = 0
    )
    
    write_w0_and_split_w0_into_lower_weights(
        g_mat = g_mat,
        ref_gen = ref_gen,
        library_name = "COSMIC"
    )
    message("Finished parsing Cosmic")
}

#' parse_ccle_genotype_data
#' 
#' Parses ccle genotype data
#' 
#' @param ccle_file Path to CCLE file on hard disk
#' @param ccle_sample_file Path to CCLE sample file
#' @param ref_gen Reference genome version
#' @import IRanges
#' @importFrom stats aggregate
#' @importFrom data.table fread
#' @return The R Table sim_list which contains the CCLE fingerprints
#' @keywords internal
parse_ccle_genotype_data = function(ccle_file, ccle_sample_file, ref_gen = "GRCH38"){
    
    library_name = "CCLE"
    
    # Only read in columns specified with subset
    subset = c(4, 5, 6, 16)
    ccle_genotype_tab = data.table::fread(
        ccle_file,
        select = subset,
        sep = ",",
        showProgress = FALSE
    )
    
    subset = c(1,2)
    ccle_sample_tab = data.table::fread(
      ccle_sample_file,
      select = subset,
      sep = ",",
      showProgress = FALSE
    )
    
    depmap_ID_macthes <- match(ccle_genotype_tab$DepMap_ID, ccle_sample_tab$DepMap_ID)
    ccle_genotype_tab$Cell_line <- ccle_sample_tab$cell_line_name[depmap_ID_macthes]
    
    cls = ccle_genotype_tab[, gsub("\\_.*", "", Cell_line)] 
    cls = str_replace(cls, paste( "_", library_name, sep = "" ), "")
    
    coords = paste(
        ccle_genotype_tab$Chromosome,
        ccle_genotype_tab$Start_position,
        ccle_genotype_tab$End_position,
        sep = "_"
    )
    c_matches = match(coords, unique(coords), nomatch = 0)
    #new_cls <<- c()
    
    message("Aggregating CCLE CCL names")
    new_cls = data.table::data.table(
      "CLS" = cls,
      "Index" = c_matches
    )
    new_cls = new_cls[,lapply(.SD, paste, sep = "", collapse= ","), by = Index]
    
    # Extract and process coordinates and CL IDs
    g_mat = GenomicRanges::GRanges(
      seqnames = ccle_genotype_tab$Chromosome,
      IRanges(
        start = ccle_genotype_tab$Start_position,
        end = ccle_genotype_tab$End_position
        )
    )
    g_mat = unique(g_mat)
    mcols(g_mat)$Member_CCLs = new_cls$CLS
    
    # write the stats part
    write_w0_and_split_w0_into_lower_weights(
        g_mat = g_mat,
        ref_gen = ref_gen,
        library_name = "CCLE"
    )
    
    message("Finished parsing CCLE")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statistics_identification.R
\name{add_penalty_statistics}
\alias{add_penalty_statistics}
\title{add_penalty_statistics}
\usage{
add_penalty_statistics(match_t, minimum_matching_mutations)
}
\arguments{
\item{match_t}{object that contains the matching variants}

\item{minimum_matching_mutations}{a numerical giving the minimum amount of
mutations that has to match between query and training sample for a 
positive prediction}
}
\value{
The updated statistics
}
\description{
Add penalty statistics to results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Parse_COSMIC_CCLE_CCLs.R
\name{parse_cosmic_genotype_data}
\alias{parse_cosmic_genotype_data}
\title{parse_cosmic_genotype_data}
\usage{
parse_cosmic_genotype_data(cosmic_file, ref_gen = "GRCH38")
}
\arguments{
\item{cosmic_file}{Path to cosmic clp file in hard disk}

\item{ref_gen}{Reference genome version}
}
\value{
The R Table sim_list which contains the CoSMIC CLP fingerprints
}
\description{
Parses cosmic genotype data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_missing_cls.R
\name{add_missing_cls}
\alias{add_missing_cls}
\title{add_missing_cls}
\usage{
add_missing_cls(res_table, dif_cls)
}
\arguments{
\item{res_table}{Table that contains the identification results}

\item{dif_cls}{Missing CLs}
}
\value{
Results table with added missing cls
}
\description{
add_missing_cls
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_custom_vcf_from_uniquorn.R
\name{remove_ccls_from_database}
\alias{remove_ccls_from_database}
\title{Remove Cancer Cell Line}
\usage{
remove_ccls_from_database(ccl_names, ref_gen = "GRCH37",
    library_name, test_mode = FALSE)
}
\arguments{
\item{ccl_names}{A character vector giving the names of the cancer cell line
identifiers to be removed. Can be one or many}

\item{ref_gen}{A character vector specifying the reference genome version.
All training sets are associated with a reference genome version. 
Default is \code{"GRCH37"}.}

\item{library_name}{Name of the library from which ccls are to be removed}

\item{test_mode}{Signifies if this is a test run}
}
\value{
Message that indicates whether the removal was succesful.
}
\description{
This function removes a cancer cell line training fingerprint (VCF file)
 from the database. The names of all training sets can 
 be seen by using the function \code{show_contained_cls}.
}
\examples{
remove_ccls_from_database(
    ccl_names = "HT29",
    ref_gen = "GRCH37",
    library_name = "CELLMINER",
    test_mode = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Show_database_information.R
\name{show_contained_variants_in_library}
\alias{show_contained_variants_in_library}
\title{All variants contained in reference library}
\usage{
show_contained_variants_in_library(
    ref_gen,
    library_name,
    mutational_weight_inclusion_threshold
)
}
\arguments{
\item{ref_gen}{a character vector specifying the reference genome 
version. All training sets are associated with a reference genome version. 
Default is \code{"GRCH37"}.}

\item{library_name}{Name of the reference library.}

\item{mutational_weight_inclusion_threshold}{Include only mutations 
with a weight of at least x. Range: 0.0 to 1.0. 1 = unique to CL. 
~0 = found in many CL samples.}
}
\value{
Returns a GenomicRanges object that contains the variants
}
\description{
This function shows all variants contained in a reference library
for a given inclusion weight. Default inclusion weight is 0 
(all variants).
}
\examples{
## Show all variants contained in reference library CELLMINER
show_contained_variants_in_library(
    ref_gen = "GRCH37",
    library_name = "CELLMINER",
    mutational_weight_inclusion_threshold = 0
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statistics_identification.R
\name{add_p_q_values_statistics}
\alias{add_p_q_values_statistics}
\title{add_p_q_values_statistics}
\usage{
add_p_q_values_statistics(
  g_query,
  match_t,
  p_value,
  ref_gen,
  minimum_matching_mutations,
  top_hits_per_library
)
}
\arguments{
\item{g_query}{IRanges object that contains the query variants}

\item{match_t}{A table that contains the nubmber of matching variants}

\item{p_value}{Threshold for the significance p-value}

\item{ref_gen}{Reference genome version}

\item{minimum_matching_mutations}{Manual lower amount of matching
mutations require for a significant match between a query and a 
reference}

\item{top_hits_per_library}{limits significant similarities to the first 
n hits}
}
\value{
R table with a statistic
}
\description{
A hypergeometric distribution-assumption allows to calculate the 
p-values for a significant or non-significant overlap in this function
}
\details{
\code{add_p_q_values_statistics} Calculates the p-values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Parse_COSMIC_CCLE_CCLs.R
\name{parse_ccle_genotype_data}
\alias{parse_ccle_genotype_data}
\title{parse_ccle_genotype_data}
\usage{
parse_ccle_genotype_data(ccle_file, ccle_sample_file, ref_gen = "GRCH38")
}
\arguments{
\item{ccle_file}{Path to CCLE file on hard disk}

\item{ccle_sample_file}{Path to CCLE sample file}

\item{ref_gen}{Reference genome version}
}
\value{
The R Table sim_list which contains the CCLE fingerprints
}
\description{
Parses ccle genotype data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Add_custom_vcfs.R
\name{add_custom_vcf_to_database}
\alias{add_custom_vcf_to_database}
\title{add_custom_vcf_to_database
This function adds the variants of parsed custom CCLs to a monet DB instance}
\usage{
add_custom_vcf_to_database(
    vcf_input_files,
    ref_gen = "GRCH37",
    library_name = "CUSTOM",
    n_threads = 1,
    test_mode = FALSE
)
}
\arguments{
\item{vcf_input_files}{a character vector containing the input vcf files.
This may be one or many vcf files.}

\item{ref_gen}{a character string specifying the reference genome version.
All training sets are associated with a reference genome version.
Default is \code{"GRCH37"}.}

\item{library_name}{a character string giving the name of the library to add the
cancer cell lines to. Default is \code{"CUSTOM"}. 
Library name will be automatically added as a suffix to the identifier.}

\item{n_threads}{an integer specifying the number of threads to be used.}

\item{test_mode}{Is this a test? Just for internal use}
}
\value{
Message wheather the adding was successful
}
\description{
add_custom_vcf_to_database
This function adds the variants of parsed custom CCLs to a monet DB instance
}
\examples{
HT29_vcf_file = system.file("extdata/HT29_TEST.vcf", package = "Uniquorn");
add_custom_vcf_to_database(
    vcf_input_files = HT29_vcf_file,
    library_name = "CELLMINER",
    ref_gen = "GRCH37",
    n_threads = 1,
    test_mode = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Show_database_information.R
\name{show_contained_ccls}
\alias{show_contained_ccls}
\title{show_contained_ccls}
\usage{
show_contained_ccls(ref_gen, verbose)
}
\arguments{
\item{ref_gen}{a character vector specifying the reference genome version.
All training sets are associated with a reference genome version.
Default is \code{"GRCH37"}.}

\item{verbose}{Should DB informations be printed?}
}
\value{
R table which contains identifiers of all cancer cell line samples
 which match the specified parameters (reference genome and library).
}
\description{
This function displays the names, amount of mutations and the overall 
weight of the mutations of all contained cancer cell line fingerprints 
for a chosen reference genome and optional library.
}
\examples{
## Show all contained cancer cell lines for reference GRCH37:
show_contained_ccls(ref_gen = "GRCH37", verbose = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{read_mutation_grange_objects}
\alias{read_mutation_grange_objects}
\title{read_mutation_grange_objects}
\usage{
read_mutation_grange_objects(
  library_name,
  mutational_weight_inclusion_threshold,
  ref_gen,
  test_mode
)
}
\arguments{
\item{library_name}{a character string giving the name of the library}

\item{mutational_weight_inclusion_threshold}{a numerical giving the
lower bound for mutational weight to be included}

\item{ref_gen}{Reference genome version. All training sets are 
associated with a reference genome version. Default: GRCH37}

\item{test_mode}{Is this a test? Just for internal use}
}
\value{
The R Table sim_list which contains the CoSMIC CLP fingerprints
}
\description{
Read the GRange object for a specific library
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Show_database_information.R
\name{show_which_ccls_contain_variant}
\alias{show_which_ccls_contain_variant}
\title{Cancer cell lines with specific variant}
\usage{
show_which_ccls_contain_variant(
    start,
    end,
    chromosome,
    ref_gen,
    library_name,
    mutational_weight_inclusion_threshold
)
}
\arguments{
\item{start}{Start coordinate}

\item{end}{Stop coordinate}

\item{chromosome}{Chromosome, 'chr' prefixes are ignored}

\item{ref_gen}{a character vector specifying the reference genome 
version. All training sets are associated with a reference genome 
version. Default is \code{"GRCH37"}.}

\item{library_name}{Name of the reference library}

\item{mutational_weight_inclusion_threshold}{Include only mutations 
with a weight of at least x. Range: 0.0 to 1.0. 1= unique to CL. 
~0 = found in many CCL samples.}
}
\value{
Returns a GenomicRanges object that contains the variant if present.
Member ccls can be found in the $Member_ccl vector
}
\description{
This function displays all cancer cell lines in the database which 
contain a specified variant. Utilizes closed interval coordinates.
}
\examples{
show_which_ccls_contain_variant(
    start = 92030762,
    end = 92030762,
    chromosome = 8,
    ref_gen = "GRCH37",
    library_name = "CELLMINER",
    mutational_weight_inclusion_threshold = 0
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Show_database_information.R
\name{read_library_names}
\alias{read_library_names}
\title{Library Name Reader}
\usage{
read_library_names(ref_gen)
}
\arguments{
\item{ref_gen}{a character vector specifying the reference genome 
version. All training sets are associated with a reference genome 
version. Default is \code{"GRCH37"}.}
}
\value{
Returns a character vector of the contained libraries
}
\description{
This function procides information on the reference library names
}
\examples{
read_library_names(ref_gen = "GRCH37")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Add_custom_vcfs.R
\name{parse_vcf_query_into_db}
\alias{parse_vcf_query_into_db}
\title{parse_vcf_query_into_db
This function adds the variants of parsed custom CCLs to a monet DB instance}
\usage{
parse_vcf_query_into_db(
  g_query,
  ref_gen = "GRCH37",
  library_name,
  test_mode = FALSE
)
}
\arguments{
\item{g_query}{a GenomicRanges object}

\item{ref_gen}{a character string specifying the reference genome version.
All training sets are associated with a reference genome version.
Default is \code{"GRCH37"}.}

\item{library_name}{a character string giving the name of the library to add
the cancer cell lines to. Default is \code{"CUSTOM"}. 
Library name will be automatically added as a suffix to the identifier.}

\item{test_mode}{Is this a test? Just for internal use}
}
\value{
Message wheather the adding was successful
}
\description{
parse_vcf_query_into_db
This function adds the variants of parsed custom CCLs to a monet DB instance
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Match_query_ccl_with_database.R
\name{match_query_ccl_to_database}
\alias{match_query_ccl_to_database}
\title{match_query_ccl_to_database}
\usage{
match_query_ccl_to_database(
  g_query,
  ref_gen = "GRCH37",
  library_name,
  mutational_weight_inclusion_threshold
)
}
\arguments{
\item{g_query}{IRanges object that contains the variants}

\item{ref_gen}{Reference genome version. All training sets are 
associated with a reference genome version. Default: GRCH37}

\item{library_name}{a character string giving the name of the library}

\item{mutational_weight_inclusion_threshold}{a numerical giving
the lower bound for mutational weight to be included}
}
\value{
The R Table sim_list which contains the CoSMIC CLP fingerprints
}
\description{
Matches query ccl to the database
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Init_and_load_identification.R
\name{init_and_load_identification}
\alias{init_and_load_identification}
\title{init_and_load_identification}
\usage{
init_and_load_identification( 
    verbose,
    ref_gen,
    vcf_file,
    output_dir
)
}
\arguments{
\item{verbose}{Print additional information}

\item{ref_gen}{Reference genome version. All training sets are 
associated with a reference genome version. Default: GRCH37}

\item{vcf_file}{Path to vcf_file}

\item{output_dir}{Output directory for identification results}
}
\value{
Three file path instances and the fingerprint
}
\description{
Initiate the analysis
Output basic information
}
\details{
\code{init_and_load_identification} parses vcf file and output 
basic information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Parser_scripts.R
\name{parse_vcf_file}
\alias{parse_vcf_file}
\title{Filter Parsed VCF Files}
\usage{
parse_vcf_file(
    vcf_file,
    ref_gen,
    library_name
)
}
\arguments{
\item{vcf_file}{character string giving the path to the vcf file
on the operating system.}

\item{ref_gen}{Reference genome version}

\item{library_name}{Name of the reference library}
}
\value{
Loci-based DNA-mutational fingerprint of the cancer cell line
 as found in the input VCF file.
}
\description{
Intern utility function. Filters the parsed VCF file for all informations
except for the start and length of variations/mutations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Create_BED_files.R
\name{create_bed_file}
\alias{create_bed_file}
\title{create_bed_file}
\usage{
create_bed_file(
match_t,
vcf_fingerprint,
output_file,
ref_gen,
manual_identifier

)
}
\arguments{
\item{match_t}{R table which contains the mutations from the 
training database for the cancer cell lines}

\item{vcf_fingerprint}{contains the mutations that are present 
in the query cancer cell line's vcf file}

\item{output_file}{Path to output file}

\item{ref_gen}{Reference genome version}

\item{manual_identifier}{Manually enter a vector of CL name(s) 
whose bed files should be created, independently from them 
passing the detection threshold}
}
\value{
Returns a message which indicates if the BED file 
creation has succeeded
}
\description{
Creates BED files from the found and not found annotated mutations
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Identify_VCF_file.R
\name{identify_vcf_file}
\alias{identify_vcf_file}
\title{identify_VCF_file}
\usage{
identify_vcf_file( 
    vcf_file,
    output_file,
    ref_gen,
    minimum_matching_mutations,
    mutational_weight_inclusion_threshold,
    write_xls,
    output_bed_file,
    top_hits_per_library,
    manual_identifier,
    verbose,
    p_value,
    confidence_score,
    n_threads,
    write_results
)
}
\arguments{
\item{vcf_file}{Input vcf file. Only one sample column allowed.}

\item{output_file}{Path of the output file. If blank, 
autogenerated as name of input file plus '_uniquorn_ident.tab' suffix.}

\item{ref_gen}{Reference genome version. All training sets are 
associated with a reference genome version. Default: GRCH37}

\item{minimum_matching_mutations}{The minimum amount of mutations that 
has to match between query and training sample for a positive prediction}

\item{mutational_weight_inclusion_threshold}{Include only mutations 
with a weight of at least x. Range: 0.0 to 1.0. 1= unique to CL. 
~0 = found in many CL samples.}

\item{write_xls}{Create identification results additionally 
as xls file for easier reading}

\item{output_bed_file}{If BED files for IGV visualization should be 
created for the Cancer Cell lines that pass the threshold}

\item{top_hits_per_library}{Limit the number of significant similarities
per library to n (default 3) many hits. Is particularrly used in contexts
when heterogeneous query and reference CCLs are being compared.}

\item{manual_identifier}{Manually enter a vector of CL 
name(s) whose bed files should be created, independently from 
them passing the detection threshold}

\item{verbose}{Print additional information}

\item{p_value}{Required p-value for identification.
Note that if you set the confidence score, the confidence score
overrides the p-value}

\item{confidence_score}{Cutoff for positive prediction between 0 and 100.
Calculated by transforming the p-value by -1 * log(p-value)
Note that if you set the confidence score, the confidence score
overrides the p-value}

\item{n_threads}{Number of threads to be used}

\item{write_results}{Write identification results to file}
}
\value{
R table with a statistic of the identification result
}
\description{
Identifies a cancer cell lines contained in a vcf file based 
on the pattern (start & length) of all contained mutations/ variations.
}
\details{
\code{identify_vcf_file} parses the vcf file and predicts 
the identity of the sample
}
\examples{
HT29_vcf_file = system.file("extdata/HT29.vcf", package = "Uniquorn");

identification = identify_vcf_file(
    vcf_file = HT29_vcf_file, 
    verbose = FALSE,
    write_results = FALSE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Show_database_information.R
\name{show_contained_variants_for_ccl}
\alias{show_contained_variants_for_ccl}
\title{Variants In Cancer Cell Line}
\usage{
show_contained_variants_for_ccl(
    name_ccl, 
    ref_gen,
    library_name,
    mutational_weight_inclusion_threshold
)
}
\arguments{
\item{name_ccl}{a character vector giving the identifier 
of the cancer cell line for which mutations will be shown.}

\item{ref_gen}{a character vector specifying the reference 
genome version. All training sets are associated with a 
reference genome version. Default is \code{"GRCH37"}.}

\item{library_name}{Name of the reference library}

\item{mutational_weight_inclusion_threshold}{Include only mutations 
with a weight of at least x. Range: 0.0 to 1.0. 1= unique to CL. 
~0 = found in many CCL samples.}
}
\value{
GenomicRanges object that contains the ccl's variants
}
\description{
This function shows all mutations present in the database 
for a selected cancer cell line and reference genome.
}
\examples{
## Show all mutations for Cancer Cell Line 'SK_OV_3'
show_contained_variants_for_ccl(
    name_ccl = "SK_OV_3",
    ref_gen = "GRCH37",
    library_name = "CELLMINER",
    mutational_weight_inclusion_threshold = 0
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_custom_vcf_from_uniquorn.R
\name{remove_library_from_database}
\alias{remove_library_from_database}
\title{Remove entire Library from Database}
\usage{
remove_library_from_database(library, ref_gen = "GRCH37", test_mode = FALSE)
}
\arguments{
\item{library}{a character vector giving the names of the library to be
removed.}

\item{ref_gen}{a character vector specifying the reference genome version.
All training sets are associated with a reference genome version. 
Default is \code{"GRCH37"}.}

\item{test_mode}{is this a test? Just for internal use.}
}
\value{
Message that indicates whether the removal was succesful.
}
\description{
This function removes a entire library from the database by removing all 
associated cancer cell line fingerprints from the database.
}
\examples{
remove_library_from_database(library = "CELLMINER",
                                ref_gen = "GRCH37",
                                test_mode = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Parse_COSMIC_CCLE_CCLs.R
\name{initiate_canonical_databases}
\alias{initiate_canonical_databases}
\title{initiate_canonical_databases}
\usage{
initiate_canonical_databases(
    cosmic_file = "CosmicCLP_MutantExport.tsv.gz",
    ccle_file = "CCLE_mutations.csv",
    ccle_sample_file = "sample_info.csv",
    ref_gen = "GRCH38"
)
}
\arguments{
\item{cosmic_file}{The path to the Cosmic CLP file. The Cosmic file 
can be obtained from "https://cancer.sanger.ac.uk/cell_lines/download" and 
should be labeled "CosmicCLP_MutantExport.tsv.gz".
Ensure that the right reference genome is used}

\item{ccle_file}{The path to the ccle DNA genotype data file. 
It should be labeled "CCLE_mutations.csv".
Ensure that the right reference genome is used}

\item{ccle_sample_file}{The path to the CCLE sample file.
It should be labeled "sample_info.csv" containing both the
DepMap ID and corresponding cell line name.}

\item{ref_gen}{Reference genome version}
}
\value{
Returns message if parsing process has succeeded
}
\description{
Parses data into r list variable
}
\examples{
initiate_canonical_databases(
    cosmic_file = "CosmicCLP_MutantExport.tsv.gz",
    ccle_file = "CCLE_mutations.csv",
    ccle_sample_file = "sample_info.csv",
    ref_gen = "GRCH38"
)
}
